#
# Class information for RCP_ComplexMethod
#
# Generated by 'genprovider' for use with ruby-sfcc

require 'rubygems'
require 'cim'

module MOF
  class RCP_ComplexMethod
    METHODS = {
      "InOut" => { :type => :uint32, :parameters => {:in => ["count", :uint32, "str", :string], :out => ["result", :string]} },
      "Size" => { :type => :uint32, :parameters => {:in => ["strA", :string]} }
    }
  end
end
