#
# Testcase for root/cimv2:RCP_ClassMethod
#
# Generated by 'genprovider' for use with cmpi-bindings-ruby
require 'rubygems'
require 'sfcc'
require 'test/unit'
require_relative "./helper"

class Test_RCP_ClassMethod < Test::Unit::TestCase
  def setup
    @client, @op = Helper.setup 'RCP_ClassMethod'
    assert @op.client
    assert_equal @client, @op.client
  end

  def teardown
    Helper.teardown
  end

  def test_registered
    cimclass = @client.get_class(@op)
    assert cimclass
  end


  def test_method_Classname
    assert_equal "RCP_ClassMethod", @op.Classname()
  end

end
