#
# Class information for RCP_ClassMethod
#
# Generated by 'genprovider' for use with ruby-sfcc

require 'rubygems'
require 'cim'

module MOF
  class RCP_ClassMethod
    METHODS = {
      "Classname" => { :type => :string }
    }
  end
end
